@@ -11,22 +11,24 @@
 #'
 # -------------------------------------------------------------------------
 #' Configuration files can be specified using a reduced subset of base R.
-#' Currently this is restricted to the following operators and functions:
+#' By defauly this is restricted to the following operators and functions:
 #'
 #' - <-, =, +, -, *, :,
-#' - as.Date (for character inputs),
+#' - $, [, [[,
+#' - $<-, [<-, [[<-,
+#' - c,
+#' - as.Date,
 #' - array, matrix,
 #' - list, data.frame,
-#' - c,
-#' - length,
-#' - seq (for numeric and date inputs),
-#' - sequence (for numeric inputs),
-#' - seq_len, seq_along,
-#' - Sys.Date and Sys.time.
+#' - Sys.Date, Sys.time,
+#' - seq, sequence and seq_len.
 #'
 #' We also enable a convenience function, `cc`, which automatically quotes input
 #' to save typing.
 #'
+#' Users can also inject their own functions in to the evaluation environment
+#' by supplying a list of [crates][carrier::crate()] as an additional argument.
+#'
 # -------------------------------------------------------------------------
 #' @param filename
 #'
@@ -38,6 +40,11 @@
 #'
 #' Not used if `as_is = TRUE`.
 #'
+#' @param crates
+#'
+#' A list of [carrier::crate] objects to inject in to the environment where the
+#' configuration file will be injected.
+#'
 #' @param as_is
 #'
 #' Should the configuration file be read in as is, without layering on top of
@@ -81,6 +88,7 @@
 load_config <- function(
     filename,
     config,
+    crates,
     ...,
     as_is = FALSE,
     default = "default"
@@ -102,6 +110,30 @@ load_config <- function(
             .abort("{.arg config} must be a string.")
     }
 
+    # check the crates
+    if (!missing(crates)) {
+        if (!is.list(crates))
+            .abort("{.arg crates} must be a named list of {.fun carrier::crate}.")
+
+        if (length(crates)) {
+            crate_names <- names(crates)
+            if (is.null(crate_names) || !all(nzchar(crate_names)) || anyDuplicated(crate_names)) {
+                .abort("{.arg crates} must be a uniquely-named list of {.fun carrier::crate}.")
+            }
+            idx <- vapply(crates, carrier::is_crate, TRUE)
+            if (!all(idx)) {
+                invalid <- match(FALSE, idx)
+                .abort(
+                    c(
+                        "{.arg crates} must be a uniquely-named list of {.fun carrier::crate}.",
+                        x = "Entry {.field {crate_names[invalid]}} has class {.class {class(crates[[invalid]])}}."
+                    )
+
+                )
+            }
+        }
+    }
+
     # check the default is valid
     if (!as_is && (!is.character(default) || length(default) != 1L || is.na(default)))
         .abort("{.arg default} must be a string.")
@@ -123,16 +155,31 @@ load_config <- function(
         c('array', 'matrix'),
         c('list', 'data.frame'),
         c('c', 'cc'),
-        'length',
+        c('[' ,'[.data.frame', '[.Date', '[.POSIXct'),
+        c('$', '[[', '[[.data.frame', '[[.Date', '[[.POSIXct'),
+        c('$<-', '$<-.data.frame'),
+        c('[<-', '[<-.data.frame', '[<-.Date', '[<-.POSIXct'),
         c('seq', 'seq.default', 'seq.int', 'seq.Date'),
-        c('sequence', 'sequence.default'),
-        c('seq_len', 'seq_along'),
+        c('sequence', 'sequence.default', 'seq_len'),
         c('Sys.Date', 'Sys.time')
     )
     allowed <- unlist(allow_list)
 
     parent <- list2env(mget(allowed, inherits = TRUE), parent = emptyenv())
 
+    # Add in any crates supplied by the user
+    if (!missing(crates)) {
+        idx <- crate_names %in% allowed
+        if (any(idx)) {
+            nm <- crate_names[idx[1L]]
+            .abort("{.val {nm}} cannot be used as a crated function name.")
+        }
+
+        for (i in seq_along(crate_names)) {
+            assign(crate_names[[i]], crates[[i]], envir = parent)
+        }
+    }
+
     # Now create an environment with the given parent
     envir <- new.env(parent = parent)
 
